@@ -1,5 +1,6 @@
 package ExtUtils::XSpp::Node::File;
 use strict;
+use warnings;
 use base 'ExtUtils::XSpp::Node';
 
 =head1 NAME
@@ -46,8 +47,6 @@ Returns the path of the file to write to (or C<-> for STDOUT).
 =cut
 
 sub file { $_[0]->{FILE} }
-sub print {
-  "\n#include <exception>\n\n\n"
-}
+sub print { return '' }
 
 1;
