@@ -68,10 +68,15 @@ sub parse {
                                     yyerror => \&ExtUtils::XSpp::Grammar::yyerror,
                                     yydebug => 0x00,
                                    );
-  if (ref($this->{DATA})) {
-    unshift @{$this->{DATA}},
-      ExtUtils::XSpp::Node::Raw->new(rows =>['#include <exception>']);
-  }
+}
+
+sub parse_type {
+    my( $class, $type ) = @_;
+    my $this = $class->new( string => "%_type{$type}" );
+
+    $this->parse;
+
+    return $this->{DATA};
 }
 
 sub include_file {
@@ -152,7 +157,7 @@ modify the parse tree before it is emitted.
 sub add_post_process_plugin {
   my( $this, %args ) = @_;
 
-  push @{$this->{PLUGINS}{POST_PROCESS}}, $args{plugin};
+  _add_plugin( $this, 'POST_PROCESS', \%args, 'post_process' );
 }
 
 sub post_process_plugins { $_[0]->{PLUGINS}{POST_PROCESS} || [] }
@@ -168,14 +173,14 @@ sub add_class_tag_plugin {
   my( $this, %args ) = @_;
   my $tag = $args{tag} || '_any_';
 
-  push @{$this->{PLUGINS}{CLASS_TAG}{$tag}}, $args{plugin};
+  _add_plugin( $this, 'CLASS_TAG', \%args, 'handle_class_tag' );
 }
 
 sub handle_class_tag_plugins {
   my( $this, $class, @args ) = @_;
 
   _handle_plugin( $this, $this->{PLUGINS}{CLASS_TAG}, 'class',
-                  'handle_class_tag', [ $class, @args ] );
+                  [ $class, @args ] );
 }
 
 =head2 ExtUtils::XSpp::Parser::add_function_tag_plugin
@@ -189,14 +194,14 @@ sub add_function_tag_plugin {
   my( $this, %args ) = @_;
   my $tag = $args{tag} || '_any_';
 
-  push @{$this->{PLUGINS}{FUNCTION_TAG}{$tag}}, $args{plugin};
+  _add_plugin( $this, 'FUNCTION_TAG', \%args, 'handle_function_tag' );
 }
 
-sub handle_function_tag_plugins {
-  my( $this, $function, @args ) = @_;
+sub handle_function_tags_plugins {
+  my( $this, $function, $tags ) = @_;
 
-  _handle_plugin( $this, $this->{PLUGINS}{FUNCTION_TAG}, 'function',
-                  'handle_function_tag', [ $function, @args ] );
+  _handle_plugins( $this, $this->{PLUGINS}{FUNCTION_TAG}, 'function',
+                   $tags, $function )
 }
 
 =head2 ExtUtils::XSpp::Parser::add_method_tag_plugin
@@ -210,14 +215,35 @@ sub add_method_tag_plugin {
   my( $this, %args ) = @_;
   my $tag = $args{tag} || '_any_';
 
-  push @{$this->{PLUGINS}{METHOD_TAG}{$tag}}, $args{plugin};
+  _add_plugin( $this, 'METHOD_TAG', \%args, 'handle_method_tag' );
 }
 
-sub handle_method_tag_plugins {
-  my( $this, $method, @args ) = @_;
+sub handle_method_tags_plugins {
+  my( $this, $method, $tags ) = @_;
 
-  _handle_plugin( $this, $this->{PLUGINS}{METHOD_TAG}, 'method',
-                  'handle_method_tag', [ $method, @args ] );
+  _handle_plugins( $this, $this->{PLUGINS}{METHOD_TAG}, 'method',
+                   $tags, $method );
+}
+
+=head2 ExtUtils::XSpp::Parser::add_argument_tag_plugin
+
+Adds the specified plugin to the list of plugins that can handle custom
+%foo annotations for an arguments.
+
+=cut
+
+sub add_argument_tag_plugin {
+  my( $this, %args ) = @_;
+  my $tag = $args{tag} || '_any_';
+
+  _add_plugin( $this, 'ARGUMENT_TAG', \%args, 'handle_argument_tag' );
+}
+
+sub handle_argument_tags_plugins {
+  my( $this, $argument, $tags ) = @_;
+
+  _handle_plugins( $this, $this->{PLUGINS}{ARGUMENT_TAG}, 'argument',
+                   $tags, $argument );
 }
 
 =head2 ExtUtils::XSpp::Parser::add_toplevel_tag_plugin
@@ -231,27 +257,60 @@ sub add_toplevel_tag_plugin {
   my( $this, %args ) = @_;
   my $tag = $args{tag} || '_any_';
 
-  push @{$this->{PLUGINS}{TOPLEVEL_TAG}{$tag}}, $args{plugin};
+  _add_plugin( $this, 'TOPLEVEL_TAG', \%args, 'handle_toplevel_tag' );
 }
 
 sub handle_toplevel_tag_plugins {
   my( $this, @args ) = @_;
 
   _handle_plugin( $this, $this->{PLUGINS}{TOPLEVEL_TAG}, 'top-level',
-                  'handle_toplevel_tag', [ undef, @args ] );
+                  [ undef, @args ] );
+}
+
+sub _add_plugin {
+  my( $this, $kind, $args, $default_method ) = @_;
+  my $entry = { plugin => $args->{plugin},
+                method => $args->{method} || $default_method,
+                };
+
+  if( $kind eq 'POST_PROCESS' ) {
+    push @{$this->{PLUGINS}{$kind}}, $entry;
+  } else {
+    push @{$this->{PLUGINS}{$kind}{$args->{tag} || '_any_'}}, $entry;
+  }
+}
+
+sub _handle_plugins {
+  my( $this, $plugins, $plugin_type, $tags, $arg ) = @_;
+  my @nodes;
+
+  foreach my $tag ( @{$tags || []} ) {
+    my $nodes = _handle_plugin( $this, $plugins, $plugin_type,
+                  [ $arg, $tag->{any},
+                    named                    => $tag->{named},
+                    positional               => $tag->{positional},
+                    any_named_arguments      => $tag->{named},
+                    any_positional_arguments => $tag->{positional},
+                    ] );
+
+    push @nodes, @$nodes;
+  }
+
+  return \@nodes;
 }
 
 sub _handle_plugin {
-  my( $this, $plugins, $plugin_type, $plugin_method, $plugin_args ) = @_;
+  my( $this, $plugins, $plugin_type, $plugin_args ) = @_;
   my $tag = $plugin_args->[1];
 
-  my $handled;
   foreach my $plugin ( @{$plugins->{$tag} || []}, @{$plugins->{_any_} || []} ) {
-    $handled ||= $plugin->$plugin_method( @$plugin_args );
-    last if $handled;
+    my $method = $plugin->{method};
+
+    my( $handled, @nodes ) = $plugin->{plugin}->$method( @$plugin_args );
+    return \@nodes if $handled;
   }
 
-  die "Unhandled $plugin_type annotation '$tag'" unless $handled;
+  die "Unhandled $plugin_type annotation '$tag'";
 }
 
 sub current_file { $_[0]->{PARSER}->YYData->{LEX}{FILE} }
