@@ -19,7 +19,7 @@ sub register_plugin {
 
 sub handle_method_tag {
     my( $self, $method, $any_tag, %args ) = @_;
-    my $name = $args{any_positional_arguments}[0];
+    my $name = $args{positional}[0];
 
     $method->set_perl_name( $name );
 
@@ -28,7 +28,7 @@ sub handle_method_tag {
 
 sub handle_function_tag {
     my( $self, $function, $any_tag, %args ) = @_;
-    my $name = $args{any_positional_arguments}[0];
+    my $name = $args{positional}[0];
 
     $function->set_perl_name( $name );
 
@@ -37,7 +37,7 @@ sub handle_function_tag {
 
 sub handle_class_tag {
     my( $self, $class, $any_tag, %args ) = @_;
-    my $name = $args{any_positional_arguments}[0];
+    my $name = $args{positional}[0];
 
     $class->set_perl_name( $name );
 
@@ -46,7 +46,7 @@ sub handle_class_tag {
 
 sub handle_toplevel_tag {
     my( $self, undef, $any_tag, %args ) = @_;
-    my $name = $args{any_positional_arguments}[0];
+    my $name = $args{positional}[0];
 
     push @{$self->{directives}}, $name;
 
